-- to see the port associated with database mirroring endpoint --
select name, type_desc, port, ip_address from sys.tcp_endpoints
select name,role_desc,state_desc from sys.database_mirroring_endpoints

ALTER DATABASE AdventureWorks 
    SET PARTNER = 'TCP://DBARCHLT:1430' <<-- is the principal's port/endpoint address
GO
--*********************************--
--***  To CREATE a new endpoint ***-- PARTNER ROLE
--*********************************--
--CREATE ENDPOINT endpoint4mirroring
--STATE = STARTED
--AS TCP( LISTENER_PORT = 1440 )
--FOR DATABASE_MIRRORING (ROLE = PARTNER);

CREATE ENDPOINT [endpoint4mirroring] 
--	AUTHORIZATION [DBARCHLT\Paul Bertucci]
	STATE=STARTED
	AS TCP (LISTENER_PORT = 1440, LISTENER_IP = ALL)
	FOR DATA_MIRRORING (ROLE = PARTNER, AUTHENTICATION = WINDOWS NEGOTIATE
, ENCRYPTION = REQUIRED ALGORITHM RC4)

--*******************************************--
-- GRANTING Connection permissions for login --
--*******************************************--
GRANT CONNECT ON ENDPOINT::endpoint4mirroring TO [DBARCHLT\Paul Bertucci];

--*************************************--
--***  To DROP an existing endpoint ***--
--*************************************--
-- DROP ENDPOINT endpoint4mirroring;

--*************************************--
--*** To ALTER an existing endpoint ***--
--*************************************--
-- ALTER ENDPOINT endpoint4mirroring
-- STATE = STARTED
-- AS TCP( LISTENER_PORT = 1440 )
-- FOR DATABASE_MIRRORING (ROLE = PARTNER);

----------------------------------------
-- BACKUP/RESTORE for Mirroring Setup --
----------------------------------------
BACKUP DATABASE [AdventureWorks] 
TO  DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.2\MSSQL\Backup\AdventureWorksLog.bak' WITH NOFORMAT, NOINIT,  
NAME = N'AdventureWorks-Full Database Backup', SKIP, NOREWIND, NOUNLOAD,  STATS = 10

BACKUP DATABASE [AdventureWorks] 
    TO DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.2\MSSQL\Backup\AdventureWorks4Mirror.bak' 
    WITH FORMAT
GO
-- then move the backup to the other mirror (partner) location --

--------------------------------------------------------------------------------------------------
-- use this restore database (with NoRecovery option) to create the mirrored version of this DB --
--------------------------------------------------------------------------------------------------
RESTORE FILELISTONLY 
   FROM DISK = 'C:\Program Files\Microsoft SQL Server\MSSQL.4\MSSQL\Backup\AdventureWorks.bak'

RESTORE DATABASE AdventureWorks 
   FROM DISK = 'C:\Program Files\Microsoft SQL Server\MSSQL.2\MSSQL\Backup\AdventureWorks4Mirror.bak'
  WITH NORECOVERY, 
        MOVE 'AdventureWorks_Data' TO 'C:\Program Files\Microsoft SQL Server\MSSQL.4\MSSQL\Data\AdventureWorks_Data.mdf',
        MOVE 'AdventureWorks_Log'  TO 'C:\Program Files\Microsoft SQL Server\MSSQL.4\MSSQL\Data\AdventureWorks_Log.ldf'
GO
-- Processed 21200 pages for database 'AdventureWorks', file 'AdventureWorks_Data' on file 1.
-- Processed 2 pages for database 'AdventureWorks', file 'AdventureWorks_Log' on file 1.
-- RESTORE DATABASE successfully processed 21202 pages in 14.677 seconds (11.833 MB/sec).
-->>>> Database status (Restoring...)


-- RESTORE FILELISTONLY 
--   FROM DISK = 'C:\Program Files\Microsoft SQL Server\MSSQL.4\MSSQL\Backup\AdventureWorksLog.bak'

-- Restore the log backups.
RESTORE LOG AdventureWorks
   FROM DISK = 'C:\Program Files\Microsoft SQL Server\MSSQL.4\MSSQL\Backup\AdventureWorks.bak'
   WITH  NORECOVERY;

Processed 0 pages for database 'AdventureWorks', file 'AdventureWorks_Data' on file 1.
Processed 2 pages for database 'AdventureWorks', file 'AdventureWorks_Log' on file 1.
RESTORE LOG successfully processed 2 pages in 0.045 seconds (0.364 MB/sec).

RESTORE LOG [AdventureWorks] 
   FROM  DISK = N'C:\Program Files\Microsoft SQL Server\MSSQL.2\MSSQL\Backup\AdventureWorksLog.bak' 
   WITH  FILE = 5, NORECOVERY 
GO
Processed 0 pages for database 'AdventureWorks', file 'AdventureWorks_Data' on file 4.
Processed 39 pages for database 'AdventureWorks', file 'AdventureWorks_Log' on file 4.
RESTORE LOG successfully processed 39 pages in 0.034 seconds (9.396 MB/sec).

-- basically, we are ready to go...
