-- Listing 23.28 - Using Output Parameters in sp_executesql
IF EXISTS ( SELECT * FROM INFORMATION_SCHEMA.ROUTINES
               WHERE SPECIFIC_SCHEMA = N'dbo'
                 AND SPECIFIC_NAME = N'get_avg_price')
   DROP PROCEDURE dbo.get_avg_price
GO
create proc get_avg_price @dbname sysname,
                          @type varchar(12) = '%'
as

declare @dsql nvarchar(500),
        @avgval float

/*********************************************************
** build the dynamic query using the @avg and @type as
** variables, which will be passed in via sp_executesql
**********************************************************/
select @dsql = 'select @avg = avg(isnull(price, 0)) from '
                + @dbname+ '..titles '
                + 'where type like @type'

--print @dsql

/*************************************************************
** submit the dynamic query using sp_executesql, passing type
**  as an input parameter, and @avgval as an output parameter
**  The value of @avg in the dynamic query will be passed
**  back into @avgval
*************************************************************/
exec sp_executesql @dsql, N'@avg float OUT, @type varchar(12)',
                   @avgval OUT, @type
print 'The avg value of price for the titles table'
      + ' where type is like ''' + @type
      + ''' in the ' + @dbname + ' database'
      + ' is ' + ltrim(str(@avgval, 9,4))

go

exec get_avg_price @dbname = 'bigpubs2005',
                   @type = 'business'
go

/*
The avg value of price for the titles table where type is like 'business'
 in the bigpubs2005 database is 15.0988
*/

exec get_avg_price @dbname = 'bigpubs2005',
                   @type = DEFAULT
go

/*
The avg value of price for the titles table where type is like '%' in the
  bigpubs2005 database is 0.3744
*/
