--Listing 23.3 - An Example of Custom Stored Procedure Template
-- =============================================
-- Create basic stored procedure template
-- =============================================

-- Drop stored procedure if it already exists
IF EXISTS (
  SELECT *
    FROM INFORMATION_SCHEMA.ROUTINES
   WHERE SPECIFIC_SCHEMA = N'<Schema_Name, sysname, dbo>'
     AND SPECIFIC_NAME = N'<Proc_Name, sysname, myproc>'
)
   DROP PROCEDURE <Schema_Name, sysname, dbo>.<Proc_Name, sysname, myproc>
GO

-- =============================================
-- Author:         <Author,,Name>
-- Create date: <Create Date,,>
-- Description:    <Description,,>
-- =============================================
CREATE PROCEDURE <Schema_Name, sysname, dbo>.<Proc_Name, sysname, myproc>
    -- Add the parameters for the stored procedure here
    <@param1, sysname, @p1> <param1_type, , int> = <param1_default, , 0>,
    <@param2, sysname, @p2> <param2_type, , int> = <param2_default, , 0>,
    <@param3, sysname, @p3> <param3_type, , int>  OUTPUT
AS
BEGIN
    -- SET NOCOUNT ON added to prevent extra result sets from
    -- interfering with SELECT statements.
    SET NOCOUNT ON;

    DECLARE @trancnt int
    SELECT @trancnt = @@TRANCOUNT

    if @trancnt = 0
        BEGIN TRAN <Proc_Name, sysname, myproc>
    else
        SAVE TRAN <Proc_Name, sysname, myproc>

    /* Insert processing code here */

    if (@@error != 0) -- check for error condition
    begin
        -- rollback to savepoint, or begin tran
        rollback tran <Proc_Name, sysname, myproc>
        -- return error code indicating rollback
        return -101
    end

    /* Insert more processing here if required */


    -- set value of output parameter
    set <@param3,sysname, @p3> = <@param1,sysname, @p1> + <@param2,sysname, @p2>

    if @trancnt = 0     -- this proc issued begin tran
      -- commit tran, decrement @@trancount to 0
      commit tran <Proc_Name, sysname, myproc>
    -- commit not required with save tran

    return 0 /* successful return */

END
GO


-- =============================================
-- Example to execute the stored procedure
-- =============================================
DECLARE <@output_variable, sysname, @p3_output> <output_datatype, , int>

EXECUTE <Schema_name, sysname, dbo>.<Proc_name, sysname, myproc>
        <@param1, sysname, @p1> = <param1_value, , 1>,
        <@param2, sysname, @p2> = <param2_value, , 1>,
        <@param3, sysname, @p3> = <@output_variable, sysname, @p3_output> OUTPUT

SELECT <@output_variable, sysname, @p3_output>
GO
