-- Listing 23.30 - Template Code for a Stored Procedure That Can Run as Part of a Transaction or Run As Its Own Transaction
/* proc to demonstrate no net change to @@trancount
** but rolls back changes within the proc
** VERY IMPORTANT: return an error code
** to tell the calling procedure rollback occurred */

create proc ptran1
as
declare @trncnt int

select @trncnt = @@trancount  -- save @@trancount value

if @trncnt = 0   -- transaction has not begun
  begin tran ptran1  -- begin tran increments nest level to 1

else             -- already in a transaction
  save tran ptran1   -- save tran doesn't increment nest level

/* do some processing */

if (@@error != 0) -- check for error condition
begin
    rollback tran ptran1  -- rollback to savepoint, or begin tran
    return 25           -- return error code indicating rollback
end

/* more processing if required */

if @trncnt = 0     -- this proc issued begin tran
  commit tran ptran1   -- commit tran, decrement @@trancount to 0
                   -- commit not required with save tran

return 0 /* successful return */
go
