--Listing 23.4 - An Example of CREATE PROCEDURE Script Generated from the Custom Stored Procedure Template
-- =============================================
-- Create basic stored procedure template
-- =============================================

-- Drop stored procedure if it already exists
IF EXISTS (
  SELECT *
    FROM INFORMATION_SCHEMA.ROUTINES
   WHERE SPECIFIC_SCHEMA = N'dbo'
     AND SPECIFIC_NAME = N'myproc'
)
   DROP PROCEDURE dbo.myproc
GO

-- =============================================
-- Author:         Name
-- Create date:
-- Description:    
-- =============================================
CREATE PROCEDURE dbo.myproc
    -- Add the parameters for the stored procedure here
    @p1 int = 0,
    @p2 int = 0,
    @p3 int  OUTPUT
AS
BEGIN
    -- SET NOCOUNT ON added to prevent extra result sets from
    -- interfering with SELECT statements.
    SET NOCOUNT ON;

    DECLARE @trancnt int
    SELECT @trancnt = @@TRANCOUNT

    if @trancnt = 0
        BEGIN TRAN myproc
    else
        SAVE TRAN myproc

    /* Insert processing code here */

    if (@@error != 0) -- check for error condition
    begin
        -- rollback to savepoint, or begin tran
        rollback tran myproc
        -- return error code indicating rollback
        return -101
    end

    /* Insert more processing here if required */


    -- set value of output parameter
    set @p3 = @p1 + @p2

    if @trancnt = 0     -- this proc issued begin tran
      -- commit tran, decrement @@trancount to 0
      commit tran myproc
    -- commit not required with save tran

    return 0 /* successful return */

END
GO


-- =============================================
-- Example to execute the stored procedure
-- =============================================
DECLARE @p3_output int

EXECUTE dbo.myproc
        @p1 = 1,
        @p2 = 1,
        @p3 = @p3_output OUTPUT

SELECT @p3_output
GO
