-- Listing 26.1 - Sample Stored Procedure and Tables for Transaction Testing
CREATE TABLE testable (col1 int)
go
CREATE TABLE auditlog (who varchar(128), valuentered int null)
go
CREATE PROCEDURE trantest @arg INT
AS
BEGIN TRAN
   IF EXISTS( SELECT * FROM testable WHERE col1 = @arg )
   BEGIN
      RAISERROR ('Value %d already exists!', 16, -1, @arg)
      ROLLBACK TRANSACTION
   END
   ELSE
   BEGIN
      INSERT INTO testable (col1) VALUES (@arg)
      COMMIT TRAN
   END

INSERT INTO auditlog (who, valuentered) VALUES (USER_NAME(), @arg)
return
go

set nocount on
EXEC trantest 1
EXEC trantest 2
SELECT * FROM testable
SELECT valuentered FROM auditlog
go

/*
col1
-----------
1
2

valuentered
-----------
1
2
*/

set nocount on
EXEC trantest 1
EXEC trantest 2
SELECT * FROM testable
SELECT valuentered FROM auditlog
go

/*
Msg 50000, Level 16, State 1, Procedure trantest, Line 6
Value 1 already exists!

Msg 50000, Level 16, State 1, Procedure trantest, Line 6
Value 2 already exists!

col1
-----------
1
2


valuentered
-----------
1
2
1
2
*/

set nocount on
BEGIN TRAN
EXEC trantest 3
EXEC trantest 1
EXEC trantest 4
COMMIT TRAN
SELECT * FROM testable
SELECT valuentered FROM auditlog
go
/*
The execution of this code gives the following results:
Msg 50000, Level 16, State 1, Procedure trantest, Line 6
Value 1 already exists!

Msg 266, Level 16, State 2, Procedure trantest, Line 0
Transaction count after EXECUTE indicates that a COMMIT or ROLLBACK TRANSACTION
 statement is missing. Previous count = 1, current count = 0.

Msg 3902, Level 16, State 1, Line 5
The COMMIT TRANSACTION request has no corresponding BEGIN TRANSACTION.

col1
-----------
1
2
4

valuentered
-----------
1
2
1
2
1
4
*/


