/***************************************************************************
** Using sys.dm_exec_query_plan
****************************************************************************/

-- obtain the plan handle (assume in this case that the SPID is 58):
select plan_handle from sys.dm_exec_requests where session_id = 58
go

/*
plan_handle
----------------------------------------------------------------------------
0x06000A00E96E6D2CB8A1F505000000000000000000000000
*/

--Once you have the plan handle, you can pass it on to the sys.dm_exec_query_plan DMV to return the query plan:
SELECT query_plan
FROM sys.dm_exec_query_plan (0x06000A00E96E6D2CB8A1F505000000000000000000000000)

--use the CROSS APPLY clause with sys.dm_exec_query_plan
SELECT query_plan FROM sys.dm_exec_requests cp
    CROSS APPLY sys.dm_exec_query_plan(cp.plan_handle)
    where cp.session_id = 58

--To return the query plan for all currently running T-SQL batches, you can run the following:
SELECT query_plan FROM sys.dm_exec_requests cp
    CROSS APPLY sys.dm_exec_query_plan(cp.plan_handle)

--The sys.dm_exec_cached_plans DMV can be used to return information about all query plans currently residing in the plan cache. 
--For example, to retrieve a snapshot of all query plans residing in the plan cache, you use the CROSS APPLY operator to 
--pass the plan handles from sys.dm_exec_cached_plans to sys.dm_exec_query_plan, as follows:
SELECT * FROM sys.dm_exec_cached_plans cp
   CROSS APPLY sys.dm_exec_query_plan(cp.plan_handle)

--To retrieve a snapshot of all query plans that currently reside in the plan cache for which the server has gathered statistics,
--  use the CROSS APPLY operator to pass the plan handles from sys.dm_exec_query_stats to sys.dm_exec_query_plan as follows:
SELECT * FROM sys.dm_exec_query_stats qs
   CROSS APPLY sys.dm_exec_query_plan(qs.plan_handle)

