--Listing 32.7 - An Example of a Procedure for Optimistic Locking
create proc optimistic_update
       @id int, -- provide the primary key for the record
       @data_field_1 varchar(10), -- provide the data value to be updated
       @timestamp timestamp -- pass in the timestamp value retrieved with
                            --  the initial data retrieval
as
-- Attempt to modify the record
update data_table
   set data_field_1 = @data_field_1
   where id = @id
     and timestamp = @timestamp
-- Check to see if no rows updated
IF @@ROWCOUNT=0
BEGIN
   if exists (SELECT * FROM data_table WHERE id=@id)
   -- The row exists but the timestamps don't match
   begin
      raiserror ('The row with id "%d" has been updated since it was read',
                  10, 1, @id)
      return -101
   end
   else  -- the row has been deleted
   begin
      raiserror ('The row with id "%d" has been deleted since it was read',
                   10, 2, @id)
      return -102
   end
end
ELSE
   PRINT 'Data Updated'
return 0
go
