using System;
using System.Data;
using System.Data.Sql;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;
//added
using System.Text.RegularExpressions;

[Serializable]
[Microsoft.SqlServer.Server.SqlUserDefinedType(
  Format.UserDefined, // requires IBinarySerialize
  IsFixedLength=false,
  IsByteOrdered=true,
  MaxByteSize=250,
  ValidationMethodName = "RegexPatternValidator"
)]
public struct RegexPattern : INullable, IBinarySerialize
{
  //instance data fields
  private Regex _reg;
  private bool _null;

  //constructor
  public RegexPattern(String Pattern)
  {
    reg = new Regex(Pattern);
    null = (Pattern == String.Empty);
  }

  //instance method
  public override string ToString()
  {
    return _reg.ToString();
  }

  //instance property
  public bool IsNull
  {
    get
    {
      if (_reg == null || _reg.ToString() == string.Empty)
      {
        return true;
      }
      else
        return false;
    }
  }

  //static method
  public static RegexPattern Null
  {
    get
    {
      RegexPattern NullInstance = new RegexPattern();
      NullInstance._null = true;
      return NullInstance;
    }
  }

  //static method
  public static RegexPattern Parse(SqlString Pattern)
  {
    if (Pattern.IsNull)
      return Null;
    else
    {
      RegexPattern u = new RegexPattern((String)Pattern);
      return u;
    }
  }

  //private instance method
  private bool RegexPatternValidator()
  {
    return (_reg.ToString() != string.Empty);
  }

  //instance method
  public Int32 Match(String Input)
  {
    Match m = _reg.Match(Input.ToString());
    if (m != null)
      return Convert.ToInt32(m.Success);
    else
      return 0;
  }

  //instance property
  public bool IsFullStringMatch
  {
    get
    {
      Match m = Regex.Match(_reg.ToString(), @"\^.+\$");
      if (m != null)
        return m.Success;
      else
        return false;
      }
    }
  
  //instance method
  [SqlMethod(
    DataAccess = DataAccessKind.None,
    IsMutator = false,
    IsPrecise = true,
    OnNullCall = false,
    SystemDataAccess = SystemDataAccessKind.None
  )]
  public Int32 MatchingGroupCount(SqlString Input)
  {
    Match m = _reg.Match(Input.ToString());
    if (m != null)
      return m.Groups.Count;
    else
      return 0;
  }

  //static method
  [SqlMethod(
    DataAccess = DataAccessKind.None,
    IsMutator = false,
    IsPrecise = true,
    OnNullCall = false,
    SystemDataAccess = SystemDataAccessKind.None
  )]
  public static bool UsesLookaheads(RegexPattern p) 
  // must be static to be called with :: syntax
  {
    Match m = Regex.Match(p.ToString(), @"\(\?[:|!].+\)");
    if (m != null)
      return m.Success;
    else
      return false;
  }

  #region IBinarySerialize Members

  public void Read(System.IO.BinaryReader r)
  {
    _reg = new Regex(r.ReadString());
  }

  public void Write(System.IO.BinaryWriter w)
  {
    w.Write(_reg.ToString());
  }

  #endregion
}