using System;
using System.Data;
using System.Data.Sql;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;

//added
using System.Text.RegularExpressions;
using System.Collections;
using System.Xml;
using System.IO;
using System.Data.SqlClient;

public class RegexLibrary
{
  [Microsoft.SqlServer.Server.SqlFunction
  (
    IsDeterministic = true,
    IsPrecise = true,
    Name = "MatchAll",
    DataAccess = DataAccessKind.None,
    SystemDataAccess = SystemDataAccessKind.None,
    FillRowMethodName = "FillMatchAll",
    TableDefinition = 
        @"MatchIndex int, 
          GroupList xml"
  )]
  public static IEnumerable MatchAll(string Input, 
      RegexPattern Expression, Int32 Options)
  {
    return new RegexReader(Input, Expression, Options);
  }

  public static void FillMatchAll(
      object row, 
      out SqlInt32 MatchIndex,
      out SqlXml GroupList)
  {
    Object[] RowArray = (Object[])row;
    MatchIndex = (SqlInt32)RowArray[0];
    GroupList = (SqlXml)RowArray[1];
  }

  public class RegexReader : IEnumerable
  {
    public String input = string.Empty;
    public RegexPattern Expression;
    public Int32 Options = int.MinValue;

    public RegexReader(String Input, RegexPattern Expression, Int32 Options)
    {
      this.input = Input;
      this.Expression = Expression;
      this.Options = Options;
    }

    //Called by SS after initialization
    public IEnumerator GetEnumerator()
    {
      return new RegexEnumerator(this);
    }
  }

  public class RegexEnumerator : IEnumerator
  {
    private Regex _rex = null;
    private Match _match = null;
    private Object[] _current = null;
    private RegexReader _reader = null;
    private int _matchIndex = 0;

    public RegexEnumerator(RegexReader Reader) 
    {
      _reader = Reader;
      Reset();
    }

    public void Reset()
    {
      _rex = null;
      _matchIndex = 0;
      _current = null;
      _rex = new Regex(_reader.Expression.ToString(), 
        (RegexOptions)_reader.Options);
      _match = _rex.Match(_reader.input);
    }

    public bool MoveNext()
    {
      if (_match.Success)
      {
        _matchIndex++;
        _current = new Object[6];
        _current[0] = (SqlInt32)_matchIndex;

        string GroupList = @"<matchlog pattern='" + _rex.ToString() + 
                  "' options='" + _rex.Options.ToString() + "' idx='" + 
                  _matchIndex.ToString() +
                  "' matchtext='" + _match.ToString() + "'>";
        
        for (int g = 1; g < _match.Groups.Count; g++)
        {
          Group grp = _match.Groups[g];
          GroupList += "<group idx='" + g.ToString() + 
                  "' text='" + grp + "'>";
          
          string CaptureList = string.Empty;
          CaptureCollection caps = grp.Captures;
          for (int c = 0; c < caps.Count; c++)
          {
            Capture cap = caps[c];
            CaptureList += "<capture idx='" + c + "' pos='" + 
              cap.Index.ToString() + "' text='" + cap + "'/>";
          }

          GroupList += CaptureList + "</group>";
        }

        GroupList += "</matchlog>";
        _current[1] = new SqlXml(
                  new XmlTextReader(
                    new StringReader(GroupList)));
        _match = _match.NextMatch();
        return true;
      }
      else
      {
        return false;
      }
    }

    public Object Current
    {
      get {
        return _current;
      }
    }
  }
}