use AdventureWorks
go
CREATE XML SCHEMA COLLECTION Sales.FeedbackSchemaCollection AS
'<?xml version="1.0"?>
<xsd:schema
    xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    targetNamespace="urn:www-samspublishing-com:examples:feedback_review_xsd"
    xmlns="urn:www-samspublishing-com:examples:feedback_review_xsd"
    elementFormDefault="qualified"
    attributeFormDefault="unqualified">
    <xsd:element name="feedback_review" type="feedbackReviewType"/>    
    <xsd:complexType name="feedbackReviewType">
            <xsd:sequence minOccurs="1" maxOccurs="unbounded">
                <xsd:element name="order" type="orderType"/>
            </xsd:sequence>
            <xsd:attribute
                name="product_id"
                type="xsd:integer"
                use="optional"/>
    </xsd:complexType>
    <xsd:complexType name="feedbackType" mixed="true">
        <xsd:attribute name="id" type="xsd:integer" use="required"/>
    </xsd:complexType>
    <xsd:complexType name="orderType">
        <xsd:choice minOccurs="0" maxOccurs="unbounded">
            <xsd:element name="customer_comment" type="feedbackType"/>
            <xsd:element name="company_response" type="feedbackType"/>
        </xsd:choice>
        <xsd:attribute name="id" type="xsd:integer" use="required"/>
        <xsd:attribute name="type" use="required">
            <xsd:simpleType>
                <xsd:restriction base="xsd:string">
                    <xsd:enumeration value="parts"/>
                    <xsd:enumeration value="product"/>
                    <xsd:enumeration value="service"/>
                </xsd:restriction>
            </xsd:simpleType>
        </xsd:attribute>        
    </xsd:complexType>
</xsd:schema>'
GO
CREATE FUNCTION Sales.fnCheckProductId
(
  @FeedbackReviewXml xml
)
RETURNS int
AS
BEGIN
  DECLARE @ProductId int
  
  SELECT @ProductId = @FeedbackReviewXml.value('
            declare namespace
                fr="urn:www-samspublishing-com:examples:feedback_review_xsd";
                /fr:feedback_review[1]/@product_id', 'int')

  RETURN @ProductId
END
GO
CREATE TABLE Sales.FeedbackReview
(
    FeedbackReviewId int IDENTITY(1, 1) NOT NULL PRIMARY KEY,
    ProductId int NULL REFERENCES Production.Product,
    FeedbackReviewXml xml (DOCUMENT Sales.FeedbackSchemaCollection) NOT NULL,
    CONSTRAINT ProductIdMatches
    CHECK (Sales.fnCheckProductId(FeedbackReviewXml) = ProductId)
)
