------------------------------------------------------
-- Add a linked server entry for an Access Database --
------------------------------------------------------
use [master]
go
EXEC sp_addlinkedserver @server='ACCESS_DATABASE_CUSTOMERS',
@srvproduct='Access', 
@provider='Microsoft.Jet.OLEDB.4.0',
@datasrc='C:\temp\CustomersPlus.mdb'
go
--------------------------------
-- Drop a linked server entry --
--------------------------------
use [master]
go
EXEC sp_dropserver 
     @server='ACCESS_DATABASE_CUSTOMERS', 
     @droplogins='droplogins'
go
-------------------------------------------------
-- See all tables within a linked server entry --
-------------------------------------------------
use [master]
go
EXEC sp_tables_ex 'ACCESS_DATABASE_CUSTOMERS'
go
TABLE_CAT TABLE_SCHEM TABLE_NAME        TABLE_TYPE   REMARKS
--------- ----------- ------------      ------------ -------
NULL      NULL        MSysAccessObjects ACCESS TABLE
NULL      NULL        MSysACEs          SYSTEM TABLE
NULL      NULL        MSysObjects       SYSTEM TABLE
NULL      NULL        MSysQueries       SYSTEM TABLE
NULL      NULL        MSysRelationships SYSTEM TABLE
NULL      NULL        SWCustomers       TABLE       

(6 row(s) affected)

use [master]
go
EXEC sp_columns_ex 'ACCESS_DATABASE_CUSTOMERS'
go
TABLE_CAT TABLE_SCH TABLE_NAME        COLUMN_NAME  TYPE_NAME  SIZE
------------------------------------- ------------ ---------- -------
NULL      NULL      MSysAccessObjects Data         VarBinary 3992
NULL      NULL      MSysAccessObjects ID           Long      10  
NULL      NULL      SWCustomers       CustomerID   VarChar   255 
NULL      NULL      SWCustomers       CompanyName  VarChar   255
NULL      NULL      SWCustomers       ContactName  VarChar   255 
NULL      NULL      SWCustomers       ContactTitle VarChar   255 
NULL      NULL      SWCustomers       Address      VarChar   255
NULL      NULL      SWCustomers       City         VarChar   255
NULL      NULL      SWCustomers       Region       VarChar   255
NULL      NULL      SWCustomers       PostalCode   Double    15 
NULL      NULL      SWCustomers       Country      VarChar   255
NULL      NULL      SWCustomers       Phone        VarChar   255
NULL      NULL      SWCustomers       Fax          VarChar   255

----------------------------------------------
-- SELECT data from the Linked Server table --
----------------------------------------------
SELECT substring(CustomerID,1,5) as 'CustID', substring(ContactName,1,18) as ContactName, 
substring(City,1,10) as 'City', substring(Country,1,15) as 'Country'
FROM ACCESS_DATABASE_CUSTOMERS...SWCustomers
ORDER BY ContactName
go
CustID ContactName        City       Country
------ ------------------ ---------- ---------------
PICCO  Adam Greifer       Los Angele USA
CHOPS  Martin Sommer      Berkeley   USA
MEREP  Thierry Gerardin   Vannes     FRANCE
BLAUS  Vilay Bertucci     Eugene     USA
HANAR  Yves Moison        Paris      FRANCE

(5 row(s) affected)
------------------------------------------------
-- Add a remote SQL Server as a linked server --
------------------------------------------------ 
use [master]
go
EXEC sp_addlinkedserver @server='DBARCHLT\SQL2005SUB',
@srvproduct='SQL Server'
go
--------------------------------
-- Drop a linked server entry --
--------------------------------
use [master]
go
EXEC sp_dropserver 
     @server='DBARCHLT\SQL2005SUB', 
     @droplogins='droplogins'
go
----------------------------------------------------
-- Selecting data from the Linked Server directly --
----------------------------------------------------
SELECT TOP 10 
       [CustomerID]
      ,[CustomerName]
      ,sum([YTDSalesTotal]) as 'YTD Sales'
  FROM [DBARCHLT\SQL2005SUB].[Unleashed].[dbo].[HotCustomerPlus]
GROUP BY [CustomerID]
        ,[CustomerName]
ORDER BY 3 desc
go
CustomerID  CustomerName                             YTD Sales
----------- ---------------------------------------- -------------
85          Roadway Bicycle Supply                   188509.608000
599         Westside Plaza                           178835.169285
433         Thorough Parts and Repair Services       168182.664000
205         Rally Master Company Inc                 146097.433161
546         Field Trip Store                         135208.616867
193         Perfect Toys                             134068.411416
697         Brakes and Gears                         125784.511476
448         Action Bicycle Specialists               122380.103796
506         Great Bikes                              122283.656936
230         Global Bike Retailers                    115936.910796

(10 row(s) affected)
-----------------------------------------
-- Adding an EXCEL linked Server entry --
-----------------------------------------
use [master]
go
EXEC sp_addlinkedserver 
  'Linked ExcelSW', /* linked server name you want to use*/
  'Jet Excel',      /* product name  can be anything */
  'Microsoft.Jet.OLEDB.4.0',  /* OLE provider name */ 
  'C:\TEMP\SWCustomers.xls',  /* datasource name */
   NULL,  /* location not needed in this case */
  'Excel 8.0',  /* Provider string if needed */
   NULL    /* catalog name if needed */
go
--------------------------------
-- Drop a linked server entry --
--------------------------------
use [master]
go
EXEC sp_dropserver 
     @server='Linked ExcelSW', 
     @droplogins='droplogins'
go
-- EXEC sp_tables_ex 'Linked ExcelNW'
-- go
-------------------------------------------------------------
-- Selecting data from Excel spreadsheet via Linked Server --
-------------------------------------------------------------
SELECT substring(CustomerID,1,5) as 'CustID', substring(ContactName,1,18) as ContactName, 
substring(City,1,10) as 'City', substring(Country,1,15) as 'Country'
FROM [Linked ExcelSW]...[SWCustomers$]
ORDER BY ContactName
go
CustID ContactName        City       Country
------ ------------------ ---------- ---------------
PICCO  Adam Greifer       Los Angele USA
BLAUS  John Monroe        Concord    USA
CHOPS  Martin Sommer      Berkeley   USA

(3 row(s) affected)
------------------------------------------------
-- To see the linked servers you have defined --
------------------------------------------------
use [master]
go
-- select * from sys.servers
select server_id, 
	   substring(name,1,26) as 'name', 
       substring(product,1,10) as 'Product',
       substring(provider,1,24) as 'Provider',
	   substring(data_source,1,25) as 'Source'
       from sys.servers
order by server_id
go
server_id   name                       Product    Provider                 Source
----------- -------------------------- ---------- ------------------------ -------------------------
0           DBARCHLT\SQL2005DEV        SQL Server SQLNCLI                  DBARCHLT\SQL2005DEV
1           DBARCHLT\SQL2005DIST       SQL Server SQLNCLI                  DBARCHLT\SQL2005DIST
2           DBARCHLT\SQL2005SUB        SQL Server SQLNCLI                  DBARCHLT\SQL2005SUB
3           ACCESS_DATABASE_CUSTOMERS  Access     Microsoft.Jet.OLEDB.4.0  C:\temp\CustomersPlus.mdb
4           Linked ExcelSW             Jet Excel  Microsoft.Jet.OLEDB.4.0  C:\TEMP\SWCustomers.xls

(5 row(s) affected)

-- OR --
exec sp_linkedservers
go
SRV_NAME                  SRV_PROVIDERNAME        SRV_PRODUCT SRV_DATASOURCE
------------------------- ----------------------- ---------- -------------------------
ACCESS_DATABASE_CUSTOMERS Microsoft.Jet.OLEDB.4.0 Access     C:\temp\CustomersPlus.mdb
DBARCHLT\SQL2005DEV       SQLNCLI                 SQL Server DBARCHLT\SQL2005DEV      
DBARCHLT\SQL2005DIST      SQLNCLI                 SQL Server DBARCHLT\SQL2005DIST     
DBARCHLT\SQL2005SUB       SQLNCLI                 SQL Server DBARCHLT\SQL2005SUB      
Linked ExcelSW            Microsoft.Jet.OLEDB.4.0 Jet Excel  C:\TEMP\SWCustomers.xls  

(5 row(s) affected)
-- sp_helpserver --
EXEC sp_helpserver @server='DBARCHLT\SQL2005SUB'
go
name                network_name        status  id connect..
------------------- ------------------- ------- -- ---------
DBARCHLT\SQL2005SUB DBARCHLT\SQL2005SUB rpc..   0  0  60
------------------------------------------------------------------------------
-- Distributed Query that unions all customers from multiple linked servers --
--          Access DB unioned with Excel Spreadsheet EXAMPLE                --
------------------------------------------------------------------------------
SELECT substring(CustomerID,1,5) as 'CustID', substring(ContactName,1,18) as ContactName, 
substring(City,1,10) as 'City', substring(Country,1,15) as 'Country'
FROM [Linked ExcelSW]...[SWCustomers$]
UNION
SELECT substring(CustomerID,1,5) as 'CustID', substring(ContactName,1,18) as ContactName, 
substring(City,1,10) as 'City', substring(Country,1,15) as 'Country'
FROM ACCESS_DATABASE_CUSTOMERS...SWCustomers
go
CustID ContactName        City       Country
------ ------------------ ---------- ---------------
BLAUS  John Monroe        Concord    USA
BLAUS  Vilay Bertucci     Eugene     USA
CHOPS  Martin Sommer      Berkeley   USA
HANAR  Yves Moison        Paris      FRANCE
MEREP  Thierry Gerardin   Vannes     FRANCE
PICCO  Adam Greifer       Los Angele USA

(6 row(s) affected)
----------------------------------------------------------------
-- Sets all local users to map to the specified linked server --
----------------------------------------------------------------
use [master]
go 
EXEC sp_addlinkedsrvlogin @rmtsrvname='DBARCHLT\SQL2005SUB',
@useself='true'
go
----------------------------------------------------------------------------
-- Allows all users to connect to the Access DB without user and passwork --
----------------------------------------------------------------------------
EXEC sp_addlinkedsrvlogin @rmtsrvname='ACCESS_DATABASE_CUSTOMERS',
@useself='false', @rmtuser=NULL, @rmtpassword=NULL
go
------------------------------------------------------
-- Show the linked server logins for linked servers --
------------------------------------------------------
EXEC sp_helplinkedsrvlogin 'ACCESS_DATABASE_CUSTOMERS'
go
Linked Server             Local Login   Is Self Mapping Remote Login
--------------------------------------------------------------------
ACCESS_DATABASE_CUSTOMERS NULL          1               NULL
DBARCHLT\SQL2005DEV       NULL          1               NULL
DBARCHLT\SQL2005DIST      NULL          1               NULL
DBARCHLT\SQL2005SUB       NULL          1               NULL
Linked ExcelSW            NULL          1               NULL

(5 row(s) affected)
--------------------------------------------------
-- Execute a stored procedure via linked server --
--------------------------------------------------
EXEC [DBARCHLT\SQL2005SUB].[master].[dbo].[sp_helpserver]
go