/***************************************************************************
** Provide Explicit Column Lists
***************************************************************************/
create table dbo.explicit_cols (a int, b int)
insert explicit_cols (a, b) values (10, 20)
go

create proc dbo.p_fetch_explicit_cols
as
declare @a int, @b int
declare c1 cursor for select * from explicit_cols
open c1
fetch c1 into @a, @b
while @@fetch_Status = 0
begin
    print 'the proc works!!'
    fetch c1 into @a, @b
end
close c1
deallocate c1
return
go

exec dbo.p_fetch_explicit_cols
go

/*
the proc works!!
*/

alter table explicit_cols add c int null
go
exec dbo.p_fetch_explicit_cols
go

/*
Msg 16924, Level 16, State 1, Procedure p_fetch_explicit_cols, Line 7
Cursorfetch: The number of variables declared in the INTO list must match that of
 selected columns.
*/

create proc p_insert_explicit_cols (@a int, @b int, @c int)
as
insert explicit_cols
   output inserted.*
   values (@a, @b, @c)
return
go

exec dbo.p_insert_explicit_cols 10, 20, 30
go

/*
a           b           c
----------- ----------- -----------
10          20          30
*/

alter table explicit_cols add d int null
go
exec dbo.p_insert_explicit_cols 10, 20, 30
go

/*
Msg 213, Level 16, State 1, Procedure p_insert_explicit_cols, Line 4
Insert Error: Column name or number of supplied values does not match table
 definition.
*/

alter proc p_insert_explicit_cols (@a int, @b int, @c int)
as
insert explicit_cols (a, b, c)
   output inserted.*
   values (@a, @b, @c)
return
go
exec dbo.p_insert_explicit_cols 11, 12, 13
go

/*
a           b           c           d
----------- ----------- ----------- -----------
11          12          13          NULL
*/


/***************************************************************************
** Qualify Object Names with Schema Name
****************************************************************************/

--create a user called testuser43 in the bigpubs2005 database:
use bigpubs2005
go
sp_addlogin testuser43, 'TestUser#43', bigpubs2005
go
create user testuser43
go
exec sp_addrolemember 'db_ddladmin', testuser43
exec sp_addrolemember 'db_datareader', testuser43
exec sp_addrolemember 'db_datawriter', testuser43
go

--Verify name of current schema
select schema_name()
go

/*
---------------------
dbo
*/

create table test43(a varchar(10) default schema_name() null)
go
insert test43 (a) values (DEFAULT)
go
select * from test43
go

/*
a
----------
dbo
*/

--create a schema43 schema and change the default schema for testuser43 to schema43
create schema schema43
go
alter user testuser43 with default_schema = schema43
go
create table test43(a varchar(10) default schema_name() null)
go
insert test43 (a) values (DEFAULT)
go
select * from test43
go

/*
a
----------
schema43
*/

alter user testuser43 with default_schema = schema43
go
select * from test43
go

/*
a
----------
schema43
*/

alter user testuser43 with default_schema = dbo
go
select * from test43
go

/*
a
----------
dbo
*/

alter user testuser43 with default_schema = dbo
go
select * from dbo.test43
go

/*
a
----------
dbo
*/

alter user testuser43 with default_schema = schema43
go
select * from dbo.test43
go

/*
a
----------
dbo
*/



/***************************************************************************
** Date Calculations
****************************************************************************/
--Calculating the First Day of Month
--"1/1/1900" is the default value for a date if an empty string ('') is used to represent a date.
select DATEDIFF(mm,'',getdate())
--using the number of months, you can add that result to "1/1/1900" to obtain the first day of the month for the given date:
select DATEADD(mm, DATEDIFF(mm,'',getdate()), '')
--Calculating the First Day of the Year
select DATEADD(yy, DATEDIFF(yy,'',getdate()), '')
--Calculating the First Day of the Quarter
select DATEADD(qq, DATEDIFF(qq,'',getdate()), '')
--Calculating Midnight for the Current Day
select DATEADD(dd, DATEDIFF(dd,'',getdate()), '')
--Calculating Monday of the Current Week
select DATEADD(wk, DATEDIFF(wk,'',getdate()), '')
--Calculating Other Dates
--to calculate the last day of the previous month for a given date
select DATEADD(dd, -1, DATEADD(mm, DATEDIFF(mm,'',getdate()), ''))
--determine the last day of the previous year
select DATEADD(dd, -1, DATEADD(yy, DATEDIFF(yy,'',getdate()), ''))
--determine the last day of the current month for a given date
select DATEADD(dd, -1, DATEADD(mm, DATEDIFF(mm,'',getdate()) + 1, ''))
--calculate the first day of the year to return the last day of the previous year:
select DATEADD(dd, -1, DATEADD(yy, DATEDIFF(yy,'',getdate()) + 1, ''))
--get the first day of the month and add six days to it:
select DATEADD(dd, 6, DATEADD(mm, DATEDIFF(mm,'',getdate()), ''))
--the first Monday of the current month. 
select DATEADD(wk, DATEDIFF(wk,'',
               DATEADD(dd, 6, DATEADD(mm, DATEDIFF(mm,'',getdate()), ''))), '')
--Converting Dates for Comparison
select title_id, pubdate from dbo.titles where pubdate = '1993-01-14'
go
select title_id, pubdate from dbo.titles
 where pubdate between '1993-01-14 00:00:00.0' and '1993-01-14 23:59:59.997'
go
select title_id, pubdate from dbo.titles
 where pubdate between '1993-01-14 00:00:00.0' and '1993-01-14 23:59:59.999'
go
declare @date datetime
set @date = '1993-01-14'
select title_id, pubdate from dbo.titles
where pubdate >= DATEADD(dd, DATEDIFF(dd,'',@date), '')
  and pubdate < DATEADD(dd, DATEDIFF(dd,'',@date) + 1, '')
go




/***************************************************************************
** Using CONTEXT_INFO
****************************************************************************/

declare @avg_price money
select @avg_price = avg(price) from dbo.titles
set context_info @avg_price
go

select convert(money, substring(context_info, 1, 8)) as AVG_PRICE
from master..sysprocesses
where spid = @@spid
go

/*AVG_PRICE
---------------------
0.3751
*/


--If you dont use a substring to specify only the first 8 bytes of the context_info column,
--SQL Server assumes that the money data is stored in the last 8 bytes and returns a result of 0
select convert(money, context_info) as AVG_PRICE
from master..sysprocesses
where spid = @@spid
go

/*
AVG_PRICE            
---------------------
0.00
*/

--For some other data types, such as char or datetime, you need to explicitly convert the data to binary
declare @max_date datetime,
        @context_info binary(128)
select @max_date = max(pubdate) from dbo.titles
select @context_info = substring(context_info, 1, 8)
                        + convert(binary(8), @max_date)
   from master..sysprocesses
   where spid = @@spid
set context_info @context_info
go

declare @avg_price money,
        @max_pubdate datetime
select @avg_price = substring(context_info, 1, 8),
       @max_pubdate = substring(context_info, 9, 8)
   from master..sysprocesses
   where spid = @@spid

select @avg_price as 'Avg Price', @max_pubdate as 'Max PubDate'
go

/*
Avg Price             Max PubDate
--------------------- -----------------------
0.3751                2006-10-03 01:50:30.840
*/


/***************************************************************************
** Working with Outer Joins
****************************************************************************/

select au_lname, au_fname, avg(royaltyper)
   from dbo.authors a left outer join dbo.titleauthor ta on a.au_id = ta.au_id
   group by au_lname, au_fname
   order by 3
go

/*
au_lname                                 au_fname             avg_royalty
---------------------------------------- -------------------- -----------
Greene                                   Morningstar          NULL
Greenfield                               Tom                  NULL
McBadden                                 Heather              NULL
Smith                                    Meander              NULL
Stringer                                 Dirk                 NULL
Gringlesby                               Burt                 30
O'Leary                                  Michael              35
Ringer                                   Anne                 37
Yokomoto                                 Akiko                40
MacFeather                               Stearns              42
Hunter                                   Sheryl               50
Dull                                     Ann                  50
Bennet                                   Abraham              60
Green                                    Marjorie             70
DeFrance                                 Michel               75
Karsen                                   Livia                75
Ringer                                   Albert               75
Panteley                                 Sylvia               100
White                                    Johnson              100
Straight                                 Dean                 100
Locksley                                 Charlene             100
Carson                                   Cheryl               100
Blotchet-Halls                           Reginald             100
del Castillo                             Innes                100
*/

select au_lname, au_fname, avg(royaltyper)
   from dbo.titleauthor ta right outer join dbo.authors a on ta.au_id = a.au_id
   group by au_lname, au_fname
   order by 3
go

select a.au_fname, p.pub_name
from dbo.authors a full outer join dbo.publishers p on a.state = p.state
go

/*
au_fname             pub_name
-------------------- ----------------------------------------
James A.             NULL
Francine du Plessix  Kumquat Technical Publishing
Jean                 NULL
E.L.                 Nordome Titles
Justin               Algodata Infosystems
April                Sidney's Books and More
Ron                  Algodata Infosystems
Jack                 NULL
Matt                 NULL
Josef                NULL
...
Albert               Ramona Publishers
NULL                 Gooseberry Titles
NULL                 Binnet & Hardley
NULL                 GGG&G
NULL                 Lucerne Publishing
NULL                 Tomato Books
NULL                 Significant Titles Company
*/


/***************************************************************************
** Generating T-SQL Statements with T-SQL
***************************************************************************/
--build the SQL statements necessary to grant EXECUTE permission to fred on all the stored procedures
select 'grant execute on ' +  name + ' to fred'
   from sys.procedures
   order by name
go

/*
--------------------------------------------------------
grant execute on byroyalty to fred
grant execute on cursor_proc to fred
grant execute on error_handler to fred
grant execute on find_books_by_type2 to fred
grant execute on gen_sequence to fred
grant execute on get_authors to fred
grant execute on get_next_item_from_queue to fred
grant execute on get_titles to fred
grant execute on p_explicit_cols to fred
grant execute on p_fetch_explicit_cols to fred
grant execute on p_insert_explicit_cols to fred
grant execute on reptq1 to fred
grant execute on reptq2 to fred
grant execute on reptq3 to fred
grant execute on SHOW_PARTS_LIST to fred
grant execute on title_authors to fred
grant execute on trantest to fred
grant execute on ytd_sales to fred
grant execute on ytd_sales2 to fred
*/
