-- Note: 
-- Execute the TSQL below, then change the USE statement to 'USE AdventureWorks
-- and execute it again. 
USE XCatMgmt
GO	
CREATE XML SCHEMA COLLECTION CatalogChangeSchema
AS 
'<?xml version="1.0"?>
<xs:schema
	targetNamespace="urn:www-samspublishing-com:examples:ssb:catalogchange"
	elementFormDefault="qualified"
	xmlns="urn:www-samspublishing-com:examples:ssb:catalogchange"
	xmlns:xs="http://www.w3.org/2001/XMLSchema">

	<xs:element name="CatalogChangeMessage">
		<xs:complexType>
			<xs:sequence maxOccurs="unbounded">
				<xs:element name="CatalogChange" type="CatalogChangeType"/>
			</xs:sequence>
		</xs:complexType>
	</xs:element>
	
	<xs:complexType name="CatalogChangeType">
		<xs:sequence>
			<xs:element name="Summary" type="xs:string"/>
			<xs:element name="Features" type="xs:string" minOccurs="0"/>
			<xs:element name="Specifications" type="xs:string" minOccurs="0"/>
		</xs:sequence>
		<xs:attribute name="SourceProductId"	type="xs:integer" use="required"/>
		<xs:attribute name="ManufacturerId"	type="xs:integer" use="required"/>
		<xs:attribute name="ChangeType">
			<xs:simpleType>
				<xs:restriction base="xs:integer">
					<xs:enumeration id="Insert" value="1"/>
					<xs:enumeration id="Update" value="2"/>
					<xs:enumeration id="Delete" value="3"/>
				</xs:restriction>
			</xs:simpleType>
		</xs:attribute>
		<xs:attribute name="Price" type="xs:decimal"/>
		<xs:attribute name="Name" type="xs:string"/>
	</xs:complexType>
</xs:schema>'
GO
CREATE MESSAGE TYPE 
[//samspublishing.com/SS2005/SSB/MessageTypes/CatalogChangeMessage]
AUTHORIZATION [SSBTestUserName]
VALIDATION = VALID_XML 
WITH SCHEMA COLLECTION CatalogChangeSchema
GO
CREATE XML SCHEMA COLLECTION GenericAcknowledgementSchema
AS
'<?xml version="1.0" encoding="utf-8" ?> 
<xs:schema
	targetNamespace="urn:www-samspublishing-com:examples:ssb:genericack"
	elementFormDefault="qualified"
	xmlns="urn:www-samspublishing-com:examples:ssb:genericack"
	xmlns:xs="http://www.w3.org/2001/XMLSchema">

	<xs:simpleType name="MsgTypeType">
		<xs:restriction base="xs:integer">
			<xs:enumeration id="SuccessMsg" value="0"/>
			<xs:enumeration id="FailureMsg" value="1"/>
			<xs:enumeration id="WarningMsg" value="2"/>
		</xs:restriction>
	</xs:simpleType>
	
	<xs:element name="Ack">
		<xs:complexType>
			<xs:sequence>
				<xs:element name="ResultMessage">
					<xs:complexType mixed="true">
						<xs:attribute 
							name="ContentId" 
							type="xs:integer" 
							use="optional"/>
						<xs:attribute 
							name="MsgType" 
							type="MsgTypeType"/>
					</xs:complexType>
				</xs:element>
			</xs:sequence>
			<xs:attribute name="ResultCode">
				<xs:simpleType>
					<xs:restriction base="xs:integer">
						<xs:enumeration id="Success" value="1"/>
						<xs:enumeration id="Failure" value="0"/>
					</xs:restriction>
				</xs:simpleType>
			</xs:attribute>
		</xs:complexType>
	</xs:element>
</xs:schema>'
GO
CREATE MESSAGE TYPE 
[//samspublishing.com/SS2005/SSB/MessageTypes/GenericAck]
AUTHORIZATION [SSBTestUserName]
VALIDATION = VALID_XML
WITH SCHEMA COLLECTION GenericAcknowledgementSchema 