CREATE PROCEDURE Production.ProductModelUpdate
USE AdventureWorks
GO
DROP PROC Production.ProductModelUpdate
GO
CREATE PROCEDURE Production.ProductModelUpdate
(
	@ProductId int,
	@NewName Name
)
AS
DECLARE @DialogHandle UNIQUEIDENTIFIER
DECLARE @CatalogChangeXml xml (DOCUMENT dbo.CatalogChangeSchema)
DECLARE @RemoteSSBGuid uniqueidentifier

-- Get the SSB guid for the target service's db
SELECT @RemoteSSBGuid = service_broker_guid
FROM sys.databases
WHERE name = 'XCatMgmt';

BEGIN TRAN;

UPDATE Production.ProductModel
SET Name = pm.Name -- change this to @NewName to actually modify the data
FROM Production.ProductModel pm
JOIN Production.Product p on 
p.ProductModelId = pm.ProductModelId
WHERE p.ProductId = @ProductId;

if @@ERROR != 0 
BEGIN
	ROLLBACK TRAN
	RAISERROR('(Initiator) Error during table update', 16, 1)
	RETURN
END;

BEGIN TRY;

WITH XMLNAMESPACES
(
'http://schemas.microsoft.com/sqlserver/2004/07/adventure-
[ic:ccc]works/ProductModelDescription' as p1,
	'http://schemas.microsoft.com/sqlserver/2004/07/adventure-
[ic:ccc]works/ProductModelWarrAndMain' as wm,
	'http://www.adventure-works.com/schemas/OtherFeatures' as wf,
	'http://www.w3.org/1999/xhtml' as html
)
SELECT 
	@CatalogChangeXml = CatalogDescription.query('
	for $ContextNode in //p1:ProductDescription,
		$SpecNode in $ContextNode/p1:Specifications,
		$FeatureNode in $ContextNode/p1:Features
	return 
		<CatalogChangeMessage 
xmlns="urn:www-samspublishing-com:examples:ssb:catalogchange">
			<CatalogChange 
				ChangeType="2" 
				Price="{sql:column("p.ListPrice")}"
				ManufacturerId="1" 
				Name="{sql:column("pm.Name")}"
				SourceProductId="{sql:column("p.ProductId")}">
				<Summary>{$ContextNode/p1:Summary/html:p/text()}</Summary>
				<Features>
					Handlebars: {$FeatureNode/wf:handlebar/text()}
					Wheels: {$FeatureNode/wf:wheel/text()}
					BikeFrame: {$FeatureNode/wf:BikeFrame/html:i/text()}
				</Features>
				<Specifications>
					Material: {$SpecNode/Material/text()}
					Color: {$SpecNode/Color/text()}
					ProductLine: {$SpecNode/ProductLine/text()}
					Style: {$SpecNode/Style/text()}
				</Specifications>
			</CatalogChange>
		</CatalogChangeMessage>
	')
FROM Production.ProductModel pm
JOIN Production.Product p 
ON pm.ProductModelId = p.ProductModelId
WHERE p.ProductId = @ProductId

END TRY 
BEGIN CATCH
	ROLLBACK TRAN
	RAISERROR('(Initiator) Error during XML production.', 16, 1)
	RETURN;
END CATCH

BEGIN DIALOG CONVERSATION @DialogHandle
FROM SERVICE 
[//samspublishing.com/SS2005/SSB/Services/CatalogChangeInitiatorService]
TO SERVICE 
'//samspublishing.com/SS2005/SSB/Services/CatalogMaintenanceService', @RemoteSSBGuid
ON CONTRACT 
[//samspublishing.com/SS2005/SSB/Contracts/BasicCatalogChangeContract]
WITH ENCRYPTION = OFF;

SEND ON CONVERSATION @DialogHandle
MESSAGE TYPE 
[//samspublishing.com/SS2005/SSB/MessageTypes/CatalogChangeMessage]
(@CatalogChangeXml)
PRINT '(Initiator) Message Sent Successfully.'
COMMIT TRAN
